package com.brilliance.mda.runtime.mda.config;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.PlatformTransactionManager;

import javax.sql.DataSource;
import java.io.IOException;
import java.util.*;

/**
 * 数据库配置
 */
@Configuration
public class DataSourceConfig {


    @Value("${spring.datasource.eibs.jdbc-url}")
    private String eibsdbUrl;

    @Value("${spring.datasource.eibs..username}")
    private String eibsusername;

    @Value("${spring.datasource.eibs..password}")
    private String eibspassword;

    @Value("${spring.datasource.eibs.driver-class-name}")
    private String eibsdriverClassName;


    /**
     * 数据源1
     * spring.datasource.db1：application.properteis中对应属性的前缀
     * @return
     */
    @Bean(name = "datasource_eibs")
   // @ConfigurationProperties(prefix = "spring.datasource.eibs")
   // @Bean(name = "db1")
   // @ConfigurationProperties(prefix = "spring.datasource.db1")
//    public DataSource dataSourceEibs() {
//        return DataSourceBuilder.create().build();
//    }

    public DataSource dataSourceEibs(){
        System.out.println("----------------主配" + eibsdbUrl);

        DruidDataSource datasource = new DruidDataSource();
        datasource.setUrl(eibsdbUrl);
        datasource.setUsername(eibsusername);
        datasource.setPassword(eibspassword);
        datasource.setDriverClassName(eibsdriverClassName);
      //  datasource.setValidationQuery(validationQuery);
       // setDruidOptions(datasource); // 设置druid数据源的属性
        return datasource;
    }

    /**
     * 数据源2
     * spring.datasource.db2：application.properteis中对应属性的前缀
     * @return
     */
    //@Bean(name = "db2")
    //@ConfigurationProperties(prefix = "spring.datasource.db2")

//    @ConfigurationProperties(prefix = "spring.dataource.swd")
//    public DataSource dataSourceSwd() {
//
//       return DataSourceBuilder.create().build();
////        System.out.println("swdDataSource="+properties);
////        return DataSourceBuilder.create(properties.getClassLoader())
////                //.type(HikariDataSource.class)
////                .driverClassName(properties.determineDriverClassName())
////                .url(properties.determineUrl())
////                .username(properties.determineUsername())
////                .password(properties.determinePassword())
////                .build();
//    }


    @Value("${spring.datasource.swd.jdbc-url}")
    private String swdUrl;

    @Value("${spring.datasource.swd..username}")
    private String swdusername;

    @Value("${spring.datasource.swd..password}")
    private String swdpassword;

    @Value("${spring.datasource.swd.driver-class-name}")
    private String swddriverClassName;

    @Bean(name = "datasource_swd")
    public DataSource dataSourceSwd() {
        DruidDataSource datasource = new DruidDataSource();
        datasource.setUrl(swdUrl);
        datasource.setUsername(swdusername);
        datasource.setPassword(swdpassword);
        datasource.setDriverClassName(swddriverClassName);
        //  datasource.setValidationQuery(validationQuery);
        // setDruidOptions(datasource); // 设置druid数据源的属性
        return datasource;
    }




    /**
     * 数据源3
     * spring.datasource.db2：application.properteis中对应属性的前缀
     * @return
     */
    //@Bean(name = "db2")
    //@ConfigurationProperties(prefix = "spring.datasource.db2")
    @Value("${spring.datasource.swd.jdbc-url}")
    private String fxdUrl;

    @Value("${spring.datasource.swd..username}")
    private String fxdusername;

    @Value("${spring.datasource.swd..password}")
    private String fxdpassword;

    @Value("${spring.datasource.swd.driver-class-name}")
    private String fxddriverClassName;

    @Bean(name = "datasource_fxd")
    public DataSource dataSourceFxd() {
        DruidDataSource datasource = new DruidDataSource();
        datasource.setUrl(fxdUrl);
        datasource.setUsername(fxdusername);
        datasource.setPassword(fxdpassword);
        datasource.setDriverClassName(fxddriverClassName);
        //  datasource.setValidationQuery(validationQuery);
        // setDruidOptions(datasource); // 设置druid数据源的属性
        return datasource;
    }
//    @ConfigurationProperties(prefix = "spring.dataource.fxd")
//    public DataSource dataSourceFxd() {
//        //System.out.println("fxdDataSource="+properties);
//
//        return DataSourceBuilder.create().build();
//
//    }

    /**
     * 动态数据源: 通过AOP在不同数据源之间动态切换
     * @return
     */
    @Primary
    @Bean(name = "dynamicDataSource")
    public DataSource dynamicDataSource() {
        DynamicDataSource dynamicDataSource = new DynamicDataSource();
        // 默认数据源
        dynamicDataSource.setDefaultTargetDataSource(dataSourceEibs());
        // 配置多数据源
        Map<Object, Object> dsMap = new HashMap<>();
        dsMap.put("eibs", dataSourceEibs());
        dsMap.put("swd", dataSourceSwd());
        dsMap.put("fxd", dataSourceFxd());

        dynamicDataSource.setTargetDataSources(dsMap);
        return dynamicDataSource;
    }

    /**
     * 配置多数据源后IOC中存在多个数据源了，事务管理器需要重新配置，不然器不知道选择哪个数据源
     * 事务管理器此时管理的数据源将是动态数据源dynamicDataSource
     * 配置@Transactional注解
     * @return
     */
    @Bean
    public PlatformTransactionManager transactionManager() {
        return new DataSourceTransactionManager(dynamicDataSource());
    }


    @Bean(name = "SqlSessionFactory_eibs")
    @Primary
    public SqlSessionFactory getEibs(@Qualifier("datasource_eibs") DataSource dataSource) throws Exception{
        SqlSessionFactoryBean sqlfb = new SqlSessionFactoryBean();
        sqlfb.setDataSource(dataSource);
//        sqlfb.setConfigLocation(new ClassPathResource("mybatis-config.xml"));
        //sqlfb.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:META-INF/orm/mda/*.xml"));
        //sqlfb.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/*.xml"));
        sqlfb.setMapperLocations(resolveMapperLocations());
        return sqlfb.getObject();
    }

//    @Bean(name = "transactionManager")
//    @Primary
//    public DataSourceTransactionManager getTransactionManager1(@Qualifier("datasource_eibs")DataSource dataSource) {
//        return new DataSourceTransactionManager(dataSource);
//    }

    @Bean(name = "eibs")
    @Primary
    public SqlSessionTemplate getSqlSessionTemplate1(@Qualifier("SqlSessionFactory_eibs")SqlSessionFactory sqlSessionFactory) throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }



//-----------------------------------------

    @Bean(name = "SqlSessionFactory_swd")
    public SqlSessionFactory getSqlSessionFactory2(@Qualifier("datasource_swd") DataSource dataSource) throws Exception{
        SqlSessionFactoryBean sqlfb = new SqlSessionFactoryBean();
        sqlfb.setDataSource(dataSource);
//        sqlfb.setConfigLocation(new ClassPathResource("mybatis-config.xml"));
     //   sqlfb.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:META-INF/orm/mda/*.xml"));
       // sqlfb.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/*.xml"));
        sqlfb.setMapperLocations(resolveMapperLocations());
        return sqlfb.getObject();
    }
//
//    @Bean(name = "transactionManager2")
//    public DataSourceTransactionManager getTransactionManager2(@Qualifier("datasource_swd")DataSource dataSource) {
//        return new DataSourceTransactionManager(dataSource);
//    }

    @Bean(name = "swd")
    public SqlSessionTemplate getSqlSessionTemplate2(@Qualifier("SqlSessionFactory_swd")SqlSessionFactory sqlSessionFactory) throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }



    @Bean(name = "SqlSessionFactory_fxd")
    public SqlSessionFactory getSqlSessionFactory3(@Qualifier("datasource_fxd") DataSource dataSource) throws Exception{
        SqlSessionFactoryBean sqlfb = new SqlSessionFactoryBean();
        sqlfb.setDataSource(dataSource);
//        sqlfb.setConfigLocation(new ClassPathResource("mybatis-config.xml"));
        sqlfb.setMapperLocations(resolveMapperLocations());
       // sqlfb.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/*.xml"));
        return sqlfb.getObject();
    }
    //
//    @Bean(name = "transactionManager3")
//    public DataSourceTransactionManager getTransactionManager3(@Qualifier("datasource_fxd")DataSource dataSource) {
//        return new DataSourceTransactionManager(dataSource);
//    }

    @Bean(name = "fxd")
    public SqlSessionTemplate getSqlSessionTemplate3(@Qualifier("SqlSessionFactory_fxd")SqlSessionFactory sqlSessionFactory) throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }


    private    Resource[] resolveMapperLocations() {

        ResourcePatternResolver resourceResolver = new PathMatchingResourcePatternResolver();
        List<String> mapperLocations = new ArrayList<>();
        mapperLocations.add("classpath:META-INF/orm/mda/*.xml");
        mapperLocations.add("classpath:mapper/*.xml");
        List<Resource> resources = new ArrayList();
        if (mapperLocations != null) {

            for (String mapperLocation : mapperLocations) {

                try {

                    Resource[] mappers = resourceResolver.getResources(mapperLocation);
                    resources.addAll(Arrays.asList(mappers));
                } catch (IOException e) {

                    // ignore
                }
            }
        }
        return resources.toArray(new Resource[resources.size()]);
    }

}
